@@ -1,43 +1,37 @@
-"""plotly wrapper to make easy plots easy to make"""
+"""Plotlywrapper: to make easy plots easy to make."""
 
-from __future__ import division
+from typing import Generator, Optional
 
 from tempfile import NamedTemporaryFile
-from collections import defaultdict
-
-# pylint: disable=redefined-builtin
-from builtins import zip
 
 import plotly.offline as py
 import plotly.graph_objs as go
+from plotly.basedatatypes import BaseTraceType  # pylint: disable=no-name-in-module,import-error
 
 import numpy as np
 import pandas as pd
 
 
-__version__ = '0.0.30'
+__version__ = '0.1.0'
 
 
-def _recursive_dict(*args):
-    recursive_factory = lambda: defaultdict(recursive_factory)
-    return defaultdict(recursive_factory, *args)
-
-
-def _labels(base='trace'):
+def _labels(base='trace') -> Generator[str, None, None]:
     i = 0
     while True:
         yield base + ' ' + str(i)
         i += 1
 
 
-def _detect_notebook():
-    """
+def _detect_notebook() -> bool:
+    """Detect if code is running in a Jupyter Notebook.
+
     This isn't 100% correct but seems good enough
 
     Returns
     -------
     bool
         True if it detects this is a notebook, otherwise False.
+
     """
     try:
         from IPython import get_ipython
@@ -47,6 +41,7 @@ def _detect_notebook():
     kernel = get_ipython()
     try:
         from spyder.utils.ipython.spyder_kernel import SpyderKernel
+
         if isinstance(kernel.kernel, SpyderKernel):
             return False
     except (ImportError, AttributeError):
@@ -54,17 +49,20 @@ def _detect_notebook():
     return isinstance(kernel, zmqshell.ZMQInteractiveShell)
 
 
-def _merge_layout(x, y):
-    z = y.copy()
-    if 'shapes' in z and 'shapes' in x:
-        x['shapes'] += z['shapes']
-    z.update(x)
-    return z
+def _merge_layout(x: go.Layout, y: go.Layout) -> go.Layout:
+    """Merge attributes from two layouts."""
+    xjson = x.to_plotly_json()
+    yjson = y.to_plotly_json()
+    if 'shapes' in yjson and 'shapes' in xjson:
+        xjson['shapes'] += yjson['shapes']
+    yjson.update(xjson)
+    return go.Layout(yjson)
 
 
 def _try_pydatetime(x):
-    """Opportunistically try to convert to pandas objects to datetimes
-    since plotly doesn't know how to handle them.
+    """Try to convert to pandas objects to datetimes.
+
+    Plotly doesn't know how to handle them.
     """
     try:
         # for datetimeindex
@@ -79,83 +77,73 @@ def _try_pydatetime(x):
     return x
 
 
-class Chart(object):
-    """
-    Plotly chart base class, usually this object will get created
-    by from a function.
+class Chart(go.Figure):
+    """Plotly chart base class.
+
+    Usually this object will get created by from a function.
     """
 
     def __init__(self, data=None, layout=None, repr_plot=True):
-        self.repr_plot = repr_plot
-        self.data = data
-        if data is None:
-            self.data = []
-        self.layout = layout
-        if layout is None:
-            layout = {}
-        self.layout = _recursive_dict(layout)
-        self.figure_ = None
+        """Create a chart."""
+        super().__init__(data=data, layout=layout)
+        self._repr_plot = repr_plot
 
     def __add__(self, other):
-        self.data += other.data
-        self.layout = _merge_layout(self.layout, other.layout)
+        """Add another chart or plot type to this chart."""
+        # pylint: disable=attribute-defined-outside-init
+        if isinstance(other, Chart):
+            self.add_traces(other.data)
+            self.layout = _merge_layout(self.layout, other.layout)
+        elif isinstance(other, BaseTraceType):
+            self.add_trace(other)
+        elif isinstance(other, go.Layout):
+            self.layout = _merge_layout(self.layout, other)
+        else:
+            raise ValueError('Cannot add {} to Chart'.format(other))
         return self
 
     def __radd__(self, other):
+        """Add another chart or plot type to this chart."""
         return self.__add__(other)
 
+    @property
+    def width(self):
+        """Width of the chart in pixels."""
+        return self.layout.width
+
+    @width.setter
     def width(self, value):
-        """Sets the width of the plot in pixels.
+        self.layout.width = value
 
-        Parameters
-        ----------
-        value : int
-            Width of the plot in pixels.
-
-        Returns
-        -------
-        Chart
-
-        """
-        self.layout['width'] = value
-        return self
+    @property
+    def height(self):
+        """Height of the chart in pixels."""
+        return self.layout.height
 
+    @height.setter
     def height(self, value):
-        """Sets the height of the plot in pixels.
-
-        Parameters
-        ----------
-        value : int
-            Height of the plot in pixels.
-
-        Returns
-        -------
-        Chart
-
-        """
-        self.layout['height'] = value
-        return self
+        self.layout.height = value
 
     def group(self):
-        """Sets bar graph display mode to "grouped".
+        """Set bar graph display mode to "grouped".
 
         Returns
         -------
         Chart
 
         """
-        self.layout['barmode'] = 'group'
+        self.layout.barmode = 'group'
         return self
 
     def stack(self):
-        """Sets bar graph display mode to "stacked".
+        """Set bar graph display mode to "stacked".
 
         Returns
         -------
         Chart
 
         """
-        self.layout['barmode'] = 'stack'
+        self.layout.barmode = 'stack'
         return self
 
     def legend(self, visible=True):
@@ -170,61 +158,38 @@ class Chart(object):
         Chart
 
         """
-        self.layout['showlegend'] = visible
+        self.layout.showlegend = visible
         return self
 
+    @property
+    def xlabel(self):
+        """Xaxis Label."""
+        return self.layout.xaxis.title
+
+    @xlabel.setter
     def xlabel(self, label):
-        """Sets the x-axis title.
+        self.layout.xaxis.title = label
 
-        Parameters
-        ----------
-        value : str
-            Label for the x-axis
+    @property
+    def ylabel(self):
+        """Left Yaxis Label."""
+        return self.layout.yaxis.title
 
-        Returns
-        -------
-        Chart
+    @ylabel.setter
+    def ylabel(self, label):
+        self.layout.yaxis.title = label
 
-        """
-        self.layout['xaxis']['title'] = label
-        return self
-
-    def ylabel(self, label, index=1):
-        """Sets the y-axis title.
-
-        Parameters
-        ----------
-        value : str
-            Label for the y-axis
-        index : int
-            Y-axis index
-
-        Returns
-        -------
-        Chart
-
-        """
-        self.layout['yaxis' + str(index)]['title'] = label
-        return self
+    @property
+    def zlabel(self):
+        """Zaxis Label."""
+        return self.layout.zaxis.title
 
+    @zlabel.setter
     def zlabel(self, label):
-        """Sets the z-axis title.
-
-        Parameters
-        ----------
-        value : str
-            Label for the z-axis
-
-        Returns
-        -------
-        Chart
-
-        """
-        self.layout['zaxis']['title'] = label
-        return self
+        self.layout.zaxis.title = label
 
     def xtickangle(self, angle):
-        """Sets the angle of the x-axis tick labels.
+        """Set the angle of the x-axis tick labels.
 
         Parameters
         ----------
@@ -240,7 +205,7 @@ class Chart(object):
         return self
 
     def ytickangle(self, angle, index=1):
-        """Sets the angle of the y-axis tick labels.
+        """Set the angle of the y-axis tick labels.
 
         Parameters
         ----------
@@ -258,7 +223,7 @@ class Chart(object):
         return self
 
     def xlabelsize(self, size):
-        """Set the size of the label
+        """Set the size of the label.
 
         Parameters
         ----------
@@ -273,7 +238,7 @@ class Chart(object):
         return self
 
     def ylabelsize(self, size, index=1):
-        """Set the size of the label
+        """Set the size of the label.
 
         Parameters
         ----------
@@ -288,7 +253,7 @@ class Chart(object):
         return self
 
     def xticksize(self, size):
-        """Set the tick font size
+        """Set the tick font size.
 
         Parameters
         ----------
@@ -303,7 +268,7 @@ class Chart(object):
         return self
 
     def yticksize(self, size, index=1):
-        """Set the tick font size
+        """Set the tick font size.
 
         Parameters
         ----------
@@ -318,7 +283,7 @@ class Chart(object):
         return self
 
     def ytickvals(self, values, index=1):
-        """Set the tick values
+        """Set the tick values.
 
         Parameters
         ----------
@@ -333,7 +298,7 @@ class Chart(object):
         return self
 
     def yticktext(self, labels, index=1):
-        """Set the tick labels
+        """Set the tick labels.
 
         Parameters
         ----------
@@ -348,7 +313,7 @@ class Chart(object):
         return self
 
     def xlim(self, low, high):
-        """Set xaxis limits
+        """Set xaxis limits.
 
         Parameters
         ----------
@@ -364,7 +329,7 @@ class Chart(object):
         return self
 
     def ylim(self, low, high, index=1):
-        """Set yaxis limits
+        """Set yaxis limits.
 
         Parameters
         ----------
@@ -381,23 +346,27 @@ class Chart(object):
         return self
 
     def xdtick(self, dtick):
-        self.layout['xaxis']['dtick'] = dtick
+        """Set the tick distance."""
+        self.layout.xaxis.dtick = dtick
         return self
 
     def ydtick(self, dtick, index=1):
+        """Set the tick distance."""
         self.layout['yaxis' + str(index)]['dtick'] = dtick
         return self
 
     def xnticks(self, nticks):
-        self.layout['xaxis']['nticks'] = nticks
+        """Set the number of ticks."""
+        self.layout.xaxis.nticks = nticks
         return self
 
     def ynticks(self, nticks, index=1):
+        """Set the number of ticks."""
         self.layout['yaxis' + str(index)]['nticks'] = nticks
         return self
 
     def yaxis_left(self, index=1):
-        """Puts the yaxis on the left hand side
+        """Put the yaxis on the left hand side.
 
         Parameters
         ----------
@@ -411,7 +380,7 @@ class Chart(object):
         self.layout['yaxis' + str(index)]['side'] = 'left'
 
     def yaxis_right(self, index=1):
-        """Puts the yaxis on the right hand side
+        """Put the yaxis on the right hand side.
 
         Parameters
         ----------
@@ -424,27 +393,28 @@ class Chart(object):
         """
         self.layout['yaxis' + str(index)]['side'] = 'right'
 
-    def title(self, string):
-        """Sets the title of the plot
+    @property
+    def title(self) -> str:
+        """Title of the chart."""
+        return self.layout.title
 
-        Parameters
-        ----------
-        string : str
-
-        Returns
-        -------
-        Chart
-
-        """
-        self.layout['title'] = string
-        return self
+    @title.setter
+    def title(self, string: str) -> None:
+        self.layout.title = string
 
     def __repr__(self):
-        if self.repr_plot:
+        """Show the chart."""
+        if self._repr_plot:
             self.show(filename=None, auto_open=False)
-        return super(Chart, self).__repr__()
+        return super().__repr__()
 
-    def show(self, filename=None, show_link=True, auto_open=True, detect_notebook=True):
+    def show(
+        self,
+        filename: Optional[str] = None,
+        show_link: bool = True,
+        auto_open: bool = True,
+        detect_notebook: bool = True,
+    ) -> None:
         """Display the chart.
 
         Parameters
@@ -470,36 +440,35 @@ class Chart(object):
             kargs['filename'] = filename
             kargs['auto_open'] = auto_open
 
-        self.figure_ = go.Figure(data=self.data, layout=go.Layout(**self.layout))
-        plot(self.figure_, show_link=show_link, **kargs)
+        # self._figure_ = go.Figure(data=self.data, layout=go.Layout(**self.layout))
+        plot(self, show_link=show_link, **kargs)
 
-    def save(self, filename=None, show_link=True, auto_open=False,
-             output='file', plotlyjs=True):
+    def save(
+        self,
+        filename: Optional[str] = None,
+        show_link: bool = True,
+        auto_open: bool = False,
+        output: str = 'file',
+        plotlyjs: bool = True,
+    ) -> str:
+        """Save the chart to an html file."""
         if filename is None:
             filename = NamedTemporaryFile(prefix='plotly', suffix='.html', delete=False).name
-        self.figure_ = go.Figure(data=self.data, layout=go.Layout(**self.layout))
         # NOTE: this doesn't work for output 'div'
-        py.plot(self.figure_, show_link=show_link, filename=filename, auto_open=auto_open,
-                output_type=output, include_plotlyjs=plotlyjs)
+        py.plot(
+            self,
+            show_link=show_link,
+            filename=filename,
+            auto_open=auto_open,
+            output_type=output,
+            include_plotlyjs=plotlyjs,
+        )
         return filename
 
-    def to_json(self):
-        """ Deprecated.
-        """
-        listdata = []
-        for data in self.data:
-            td = {}
-            for k, v in data.items():
-                try:
-                    td[k] = v.tolist()
-                except (AttributeError, TypeError):
-                    td[k] = v
-            listdata.append(td)
-        return dict(data=listdata, layout=self.layout)
-
     @property
     def dict(self):
-        return dict(data=self.data, layout=self.layout)
+        """Convert Chart to a dict."""
+        return self.to_dict()
 
 
 def spark_shape(points, shapes, fill=None, color='blue', width=5, yindex=0, heights=None):
@@ -517,9 +486,7 @@ def spark_shape(points, shapes, fill=None, color='blue', width=5, yindex=0, heig
 
     """
     assert len(points) == len(shapes) + 1
-    data = [{'marker': {'color': 'white'},
-             'x': [points[0], points[-1]],
-             'y': [yindex, yindex]}]
+    data = [{'marker': {'color': 'white'}, 'x': [points[0], points[-1]], 'y': [yindex, yindex]}]
 
     if fill is None:
         fill = [False] * len(shapes)
@@ -536,13 +503,17 @@ def spark_shape(points, shapes, fill=None, color='blue', width=5, yindex=0, heig
         else:
             fillcolor = 'white'
         lays.append(
-            dict(type=shape,
-                 x0=points[i], x1=points[i+1],
-                 y0=yindex - height, y1=yindex + height,
-                 xref='x', yref='y',
-                 fillcolor=fillcolor,
-                 line=dict(color=color,
-                           width=width))
+            dict(
+                type=shape,
+                x0=points[i],
+                x1=points[i + 1],
+                y0=yindex - height,
+                y1=yindex + height,
+                xref='x',
+                yref='y',
+                fillcolor=fillcolor,
+                line=dict(color=color, width=width),
+            )
         )
 
     layout = dict(shapes=lays)
@@ -573,11 +544,9 @@ def vertical(x, ymin=0, ymax=1, color=None, width=None, dash=None, opacity=None)
     if dash:
         lineattr['dash'] = dash
 
-    layout = dict(shapes=[dict(type='line',
-                               x0=x, x1=x,
-                               y0=ymin, y1=ymax,
-                               opacity=opacity,
-                               line=lineattr)])
+    layout = dict(
+        shapes=[dict(type='line', x0=x, x1=x, y0=ymin, y1=ymax, opacity=opacity, line=lineattr)]
+    )
     return Chart(layout=layout)
 
 
@@ -604,17 +573,26 @@ def horizontal(y, xmin=0, xmax=1, color=None, width=None, dash=None, opacity=Non
     if dash:
         lineattr['dash'] = dash
 
-    layout = dict(shapes=[dict(type='line',
-                               x0=xmin, x1=xmax,
-                               y0=y, y1=y,
-                               opacity=opacity,
-                               line=lineattr)])
+    layout = dict(
+        shapes=[dict(type='line', x0=xmin, x1=xmax, y0=y, y1=y, opacity=opacity, line=lineattr)]
+    )
     return Chart(layout=layout)
 
 
-def line(x=None, y=None, label=None, color=None, width=None, dash=None, opacity=None,
-         mode='lines+markers', yaxis=1, fill=None, text="",
-         markersize=6):
+def line(
+    x=None,
+    y=None,
+    label=None,
+    color=None,
+    width=None,
+    dash=None,
+    opacity=None,
+    mode='lines+markers',
+    yaxis=1,
+    fill=None,
+    text="",
+    markersize=6,
+):
     """Draws connected dots.
 
     Parameters
@@ -653,20 +631,46 @@ def line(x=None, y=None, label=None, color=None, width=None, dash=None, opacity=
                 label = _labels()
             else:
                 label = _labels(label)
-        data = [go.Scatter(x=x, y=yy, name=ll, line=lineattr, mode=mode, text=text,
-                           fill=fill, opacity=opacity, yaxis=yn, marker=dict(size=markersize))
-                for ll, yy in zip(label, y.T)]
+        data = [
+            go.Scatter(
+                x=x,
+                y=yy,
+                name=ll,
+                line=lineattr,
+                mode=mode,
+                text=text,
+                fill=fill,
+                opacity=opacity,
+                yaxis=yn,
+                marker=dict(size=markersize),
+            )
+            for ll, yy in zip(label, y.T)
+        ]
     else:
-        data = [go.Scatter(x=x, y=y, name=label, line=lineattr, mode=mode, text=text,
-                           fill=fill, opacity=opacity, yaxis=yn, marker=dict(size=markersize))]
+        data = [
+            go.Scatter(
+                x=x,
+                y=y,
+                name=label,
+                line=lineattr,
+                mode=mode,
+                text=text,
+                fill=fill,
+                opacity=opacity,
+                yaxis=yn,
+                marker=dict(size=markersize),
+            )
+        ]
     if yaxis == 1:
         return Chart(data=data)
 
     return Chart(data=data, layout={'yaxis' + str(yaxis): dict(overlaying='y')})
 
 
-def line3d(x, y, z, label=None, color=None, width=None, dash=None, opacity=None,
-           mode='lines+markers'):
+def line3d(
+    x, y, z, label=None, color=None, width=None, dash=None, opacity=None, mode='lines+markers'
+):
+    """Create a 3d line chart."""
     x = np.atleast_1d(x)
     y = np.atleast_1d(y)
     z = np.atleast_1d(z)
@@ -685,18 +689,17 @@ def line3d(x, y, z, label=None, color=None, width=None, dash=None, opacity=None,
                 label = _labels()
             else:
                 label = _labels(label)
-        data = [go.Scatter3d(x=xx, y=yy, z=zz, name=ll, line=lineattr, mode=mode,
-                             opacity=opacity)
-                for ll, xx, yy, zz in zip(label, x.T, y.T, z.T)]
+        data = [
+            go.Scatter3d(x=xx, y=yy, z=zz, name=ll, line=lineattr, mode=mode, opacity=opacity)
+            for ll, xx, yy, zz in zip(label, x.T, y.T, z.T)
+        ]
     else:
-        data = [go.Scatter3d(x=x, y=y, z=z, name=label, line=lineattr, mode=mode,
-                             opacity=opacity)]
+        data = [go.Scatter3d(x=x, y=y, z=z, name=label, line=lineattr, mode=mode, opacity=opacity)]
     return Chart(data=data)
 
 
-def scatter3d(x, y, z, label=None, color=None, width=None, dash=None, opacity=None,
-              mode='markers'):
-    """3D Scatter Plot
+def scatter3d(x, y, z, label=None, color=None, width=None, dash=None, opacity=None, mode='markers'):
+    """Create a 3D scatter Plot.
 
     Parameters
     ----------
@@ -733,17 +736,29 @@ def scatter3d(x, y, z, label=None, color=None, width=None, dash=None, opacity=No
                 label = _labels()
             else:
                 label = _labels(label)
-        data = [go.Scatter3d(x=xx, y=yy, z=zz, name=ll, line=lineattr, mode=mode,
-                             opacity=opacity)
-                for ll, xx, yy, zz in zip(label, x.T, y.T, z.T)]
+        data = [
+            go.Scatter3d(x=xx, y=yy, z=zz, name=ll, line=lineattr, mode=mode, opacity=opacity)
+            for ll, xx, yy, zz in zip(label, x.T, y.T, z.T)
+        ]
     else:
-        data = [go.Scatter3d(x=x, y=y, z=z, name=label, line=lineattr, mode=mode,
-                             opacity=opacity)]
+        data = [go.Scatter3d(x=x, y=y, z=z, name=label, line=lineattr, mode=mode, opacity=opacity)]
     return Chart(data=data)
 
 
-def scatter(x=None, y=None, label=None, color=None, width=None, dash=None, opacity=None,
-            markersize=6, yaxis=1, fill=None, text="", mode='markers'):
+def scatter(
+    x=None,
+    y=None,
+    label=None,
+    color=None,
+    width=None,
+    dash=None,
+    opacity=None,
+    markersize=6,
+    yaxis=1,
+    fill=None,
+    text="",
+    mode='markers',
+):
     """Draws dots.
 
     Parameters
@@ -757,12 +772,24 @@ def scatter(x=None, y=None, label=None, color=None, width=None, dash=None, opaci
     Chart
 
     """
-    return line(x=x, y=y, label=label, color=color, width=width, dash=dash, opacity=opacity,
-                mode=mode, yaxis=yaxis, fill=fill, text=text, markersize=markersize)
+    return line(
+        x=x,
+        y=y,
+        label=label,
+        color=color,
+        width=width,
+        dash=dash,
+        opacity=opacity,
+        mode=mode,
+        yaxis=yaxis,
+        fill=fill,
+        text=text,
+        markersize=markersize,
+    )
 
 
 def bar(x=None, y=None, label=None, mode='group', yaxis=1, opacity=None):
-    """Create a bar chart
+    """Create a bar chart.
 
     Parameters
     ----------
@@ -801,12 +828,11 @@ def bar(x=None, y=None, label=None, mode='group', yaxis=1, opacity=None):
     if yaxis == 1:
         return Chart(data=data, layout={'barmode': mode})
 
-    return Chart(data=data, layout={'barmode': mode,
-                                    'yaxis' + str(yaxis): dict(overlaying='y')})
+    return Chart(data=data, layout={'barmode': mode, 'yaxis' + str(yaxis): dict(overlaying='y')})
 
 
 def heatmap(z, x=None, y=None, colorscale='Viridis'):
-    """Create a heatmap
+    """Create a heatmap.
 
     Parameters
     ----------
@@ -822,17 +848,21 @@ def heatmap(z, x=None, y=None, colorscale='Viridis'):
 
     """
     z = np.atleast_1d(z)
-    data = [go.Heatmap(
-        z=z,
-        x=x,
-        y=y,
-        colorscale=colorscale
-    )]
+    data = [go.Heatmap(z=z, x=x, y=y, colorscale=colorscale)]
     return Chart(data=data)
 
 
-def fill_zero(x=None, y=None, label=None, color=None, width=None, dash=None, opacity=None,
-              mode='lines+markers', **kargs):
+def fill_zero(
+    x=None,
+    y=None,
+    label=None,
+    color=None,
+    width=None,
+    dash=None,
+    opacity=None,
+    mode='lines+markers',
+    **kargs
+):
     """Fill to zero.
 
     Parameters
@@ -846,13 +876,33 @@ def fill_zero(x=None, y=None, label=None, color=None, width=None, dash=None, opa
     Chart
 
     """
-    return line(x=x, y=y, label=label, color=color, width=width, dash=dash,
-                opacity=opacity, mode=mode, fill='tozeroy', **kargs)
+    return line(
+        x=x,
+        y=y,
+        label=label,
+        color=color,
+        width=width,
+        dash=dash,
+        opacity=opacity,
+        mode=mode,
+        fill='tozeroy',
+        **kargs
+    )
 
 
-def fill_between(x=None, ylow=None, yhigh=None, label=None, color=None, width=None, dash=None,
-                 opacity=None, mode='lines+markers', **kargs):
-    """Fill between `ylow` and `yhigh`
+def fill_between(
+    x=None,
+    ylow=None,
+    yhigh=None,
+    label=None,
+    color=None,
+    width=None,
+    dash=None,
+    opacity=None,
+    mode='lines+markers',
+    **kargs
+):
+    """Fill between `ylow` and `yhigh`.
 
     Parameters
     ----------
@@ -865,10 +915,30 @@ def fill_between(x=None, ylow=None, yhigh=None, label=None, color=None, width=No
     Chart
 
     """
-    plot = line(x=x, y=ylow, label=label, color=color, width=width, dash=dash,
-                opacity=opacity, mode=mode, fill=None, **kargs)
-    plot += line(x=x, y=yhigh, label=label, color=color, width=width, dash=dash,
-                 opacity=opacity, mode=mode, fill='tonexty', **kargs)
+    plot = line(
+        x=x,
+        y=ylow,
+        label=label,
+        color=color,
+        width=width,
+        dash=dash,
+        opacity=opacity,
+        mode=mode,
+        fill=None,
+        **kargs
+    )
+    plot += line(
+        x=x,
+        y=yhigh,
+        label=label,
+        color=color,
+        width=width,
+        dash=dash,
+        opacity=opacity,
+        mode=mode,
+        fill='tonexty',
+        **kargs
+    )
     return plot
 
 
@@ -888,21 +958,31 @@ def rug(x, label=None, opacity=None):
     """
     x = _try_pydatetime(x)
     x = np.atleast_1d(x)
-    data = [go.Scatter(x=x, y=np.ones_like(x), name=label,
-                       opacity=opacity,
-                       mode='markers',
-                       marker=dict(symbol='line-ns-open'))]
-    layout = dict(barmode='overlay',
-                  hovermode='closest',
-                  legend=dict(traceorder='reversed'),
-                  xaxis1=dict(zeroline=False),
-                  yaxis1=dict(domain=[0.85, 1],
-                              showline=False,
-                              showgrid=False,
-                              zeroline=False,
-                              anchor='free',
-                              position=0.0,
-                              showticklabels=False))
+    data = [
+        go.Scatter(
+            x=x,
+            y=np.ones_like(x),
+            name=label,
+            opacity=opacity,
+            mode='markers',
+            marker=dict(symbol='line-ns-open'),
+        )
+    ]
+    layout = dict(
+        barmode='overlay',
+        hovermode='closest',
+        legend=dict(traceorder='reversed'),
+        xaxis1=dict(zeroline=False),
+        yaxis1=dict(
+            domain=[0.85, 1],
+            showline=False,
+            showgrid=False,
+            zeroline=False,
+            anchor='free',
+            position=0.0,
+            showticklabels=False,
+        ),
+    )
     return Chart(data=data, layout=layout)
 
 
@@ -983,10 +1063,10 @@ def hist2d(x, y, label=None, opacity=None):
     return Chart(data=data)
 
 
-class PandasPlotting(object):
-    """
-    These plotting tools can be accessed through dataframe instance
-    accessor `.plotly`.
+@pd.api.extensions.register_dataframe_accessor('plotly')
+@pd.api.extensions.register_series_accessor('plotly')
+class PandasPlotting:
+    """Pandas plotly charting methods.
 
     Examples
     --------
@@ -999,23 +1079,44 @@ class PandasPlotting(object):
     """
 
     def __init__(self, data):
+        """Create the pandas accessor."""
         self._data = data
         if isinstance(data, pd.DataFrame):
             self._label = data.columns
         elif isinstance(data, pd.Series):
             self._label = data.name
 
-    def line(self, label=None, color=None, width=None, dash=None,
-             opacity=None, mode='lines+markers', fill=None, **kargs):
+    def line(
+        self,
+        label=None,
+        color=None,
+        width=None,
+        dash=None,
+        opacity=None,
+        mode='lines+markers',
+        fill=None,
+        **kargs
+    ):
+        """Create a line chart."""
         if label is None:
             label = self._label
-        return line(x=self._data.index, y=self._data.values, label=label,
-                    color=color, width=width, dash=dash, opacity=opacity, mode=mode,
-                    fill=fill, **kargs)
+        return line(
+            x=self._data.index,
+            y=self._data.values,
+            label=label,
+            color=color,
+            width=width,
+            dash=dash,
+            opacity=opacity,
+            mode=mode,
+            fill=fill,
+            **kargs
+        )
 
-    def scatter(self, label=None, color=None, width=None, dash=None,
-                opacity=None, mode='markers', **kargs):
-        """Creates a bar chart.
+    def scatter(
+        self, label=None, color=None, width=None, dash=None, opacity=None, mode='markers', **kargs
+    ):
+        """Create a bar chart.
 
         Parameters
         ----------
@@ -1030,11 +1131,20 @@ class PandasPlotting(object):
         """
         if label is None:
             label = self._label
-        return scatter(x=self._data.index, y=self._data.values, label=label,
-                       color=color, width=width, dash=dash, opacity=opacity, mode=mode, **kargs)
+        return scatter(
+            x=self._data.index,
+            y=self._data.values,
+            label=label,
+            color=color,
+            width=width,
+            dash=dash,
+            opacity=opacity,
+            mode=mode,
+            **kargs
+        )
 
     def bar(self, label=None, mode='group', opacity=None, **kargs):
-        """Creates a bar chart.
+        """Create a bar chart.
 
         Parameters
         ----------
@@ -1050,11 +1160,17 @@ class PandasPlotting(object):
         """
         if label is None:
             label = self._label
-        return bar(x=self._data.index, y=self._data.values, label=label,
-                   mode=mode, opacity=opacity, **kargs)
+        return bar(
+            x=self._data.index,
+            y=self._data.values,
+            label=label,
+            mode=mode,
+            opacity=opacity,
+            **kargs
+        )
 
     def stack(self, mode='lines', label=None, **kargs):
-        """Creates a stacked area plot.
+        """Create a stacked area plot.
 
         Parameters
         ----------
@@ -1074,8 +1190,15 @@ class PandasPlotting(object):
         cum = self._data.cumsum(axis=1)
         chart = Chart()
         for lab, (_, ser), (_, orig) in zip(label, cum.iteritems(), self._data.iteritems()):
-            chart += line(x=ser.index, y=ser.values, label=lab,
-                          fill='tonexty', mode=mode, text=orig.values, **kargs)
+            chart += line(
+                x=ser.index,
+                y=ser.values,
+                label=lab,
+                fill='tonexty',
+                mode=mode,
+                text=orig.values,
+                **kargs
+            )
         return chart
 
     def sparklines(self, label=None, mode='lines', percent=90, epsilon=1e-3):
@@ -1098,7 +1221,7 @@ class PandasPlotting(object):
         div = self._data.max(axis=0) - self._data.min(axis=0) + epsilon
         center = div / 2. + self._data.min(axis=0)
         normed = (self._data - center) / div
-        normed *= (percent / 100.)
+        normed *= percent / 100.
         offset = np.arange(1, self._data.shape[1] + 1)
         normed += offset
 
@@ -1107,29 +1230,3 @@ class PandasPlotting(object):
         chart.yticktext(self._data.columns.values)
         chart.legend(False)
         return chart
-
-
-# pylint: disable=too-few-public-methods
-class _AccessorProperty(object):
-    """Descriptor for implementing accessor properties.
-    Borrowed from pandas.
-    """
-    def __init__(self, accessor_cls, construct_accessor):
-        self.accessor_cls = accessor_cls
-        self.construct_accessor = construct_accessor
-        self.__doc__ = accessor_cls.__doc__
-
-    def __get__(self, instance, owner=None):
-        if instance is None:
-            return self.accessor_cls
-        return self.construct_accessor(instance)
-
-    def __set__(self, instance, value):
-        raise AttributeError("can't set attribute")
-
-    def __delete__(self, instance):
-        raise AttributeError("can't delete attribute")
-
-
-pd.DataFrame.plotly = _AccessorProperty(PandasPlotting, PandasPlotting)
-pd.Series.plotly = _AccessorProperty(PandasPlotting, PandasPlotting)
